﻿using System;
using System.Data;
using System.Web.UI.WebControls;
using CustomPaging.Class;

namespace CustomPaging.WebForms
{
    public partial class DemoForm : System.Web.UI.Page
    {
        #region Variables
        private const string SelectQuery = "Select * From TmpData";
        private const string RecCountQuery = "Select Count(*) From TmpData";
        private int _totalPages;
        private int _curPage;
        private int _totalRecords;
        #endregion

        #region Controls
        protected void Page_PreInit(object sender, EventArgs e)
        {
            CreateControls();
        }
        protected void Page_Load(object sender, EventArgs e)
        {
            SetIndex();
            if(!Page.IsPostBack)
            {
                BindData(0);
                BindDataTable();
                EnableDisableLink(1);
            }            
        }
        protected void grdEmployee_RowCreated(object sender, GridViewRowEventArgs e)
            {
                if (e.Row.RowType == DataControlRowType.Pager)
                {
                    e.Row.Visible = false;
                }
            }
        protected void grdEmpDT_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {            
            grdEmpDT.PageIndex = e.NewPageIndex;
            BindDataTable();
            
            //Call the First Grid Navigate on the Navigation.
            CommandEventArgs temp;
            ViewState["CurIndex"] = e.NewPageIndex +1;
            BindData(e.NewPageIndex);
            EnableDisableLink(e.NewPageIndex + 1);
        }
        private void CreateControls()
        {
            LinkButton lnkFirst = new LinkButton();
            lnkFirst.ID = "lnkFirst";
            lnkFirst.Text = "First";
            lnkFirst.CommandName = "First";
            lnkFirst.Command += Navigate;

            Literal ltlSpc1 = new Literal();
            ltlSpc1.ID = "ltlSpc1";
            ltlSpc1.Text = "  ";

            LinkButton lnkPrevious = new LinkButton();
            lnkPrevious.ID = "lnkPrev";
            lnkPrevious.Text = "Prev";
            lnkPrevious.CommandName = "Prev";
            lnkPrevious.Command += Navigate;

            Literal ltlSpc2 = new Literal();
            ltlSpc2.ID = "ltlSpc2";
            ltlSpc2.Text = "  ";

            LinkButton lnkNext = new LinkButton();
            lnkNext.ID = "lnkNext";
            lnkNext.Text = "Next";
            lnkNext.CommandName = "Next";
            lnkNext.Command += Navigate;

            Literal ltlSpc3 = new Literal();
            ltlSpc3.ID = "ltlSpc3";
            ltlSpc3.Text = "  ";

            LinkButton lnkLast = new LinkButton();
            lnkLast.ID = "lnkLast";
            lnkLast.Text = "Last";
            lnkLast.CommandName = "Last";
            lnkLast.Command += Navigate;

            plcControls.Controls.Clear();
            plcControls.Controls.Add(lnkFirst);
            plcControls.Controls.Add(ltlSpc1);
            plcControls.Controls.Add(lnkPrevious);
            plcControls.Controls.Add(ltlSpc2);
            plcControls.Controls.Add(lnkNext);
            plcControls.Controls.Add(ltlSpc3);
            plcControls.Controls.Add(lnkLast);
            }
        private void SetIndex()
        {            
            _totalRecords= GetRecdCount();
            _totalPages = (int)Math.Ceiling((double)_totalRecords / (double)grdEmployee.PageSize);
            if (ViewState["CurIndex"] == null)
            {
                _curPage = 1;
            }
            else
            {
                _curPage = (int)ViewState["CurIndex"];
            }
            LinkButton lnkButton;
            lnkButton = (LinkButton) plcControls.FindControl("lnkFirst");
            if (lnkButton != null) lnkButton.CommandArgument = "1";

            lnkButton = (LinkButton)plcControls.FindControl("lnkPrev");
            if (lnkButton != null) lnkButton.CommandArgument = (_curPage - 1).ToString();

            lnkButton = (LinkButton)plcControls.FindControl("lnkNext");
            if (lnkButton != null) lnkButton.CommandArgument = (_curPage + 1).ToString();

            lnkButton = (LinkButton)plcControls.FindControl("lnkLast");
            if (lnkButton != null) lnkButton.CommandArgument = _totalPages.ToString();

        }
        private void EnableDisableLink(int curIndex)
        {
            LinkButton lnkFirst = (LinkButton) plcControls.FindControl("lnkFirst");
            LinkButton lnkPrev = (LinkButton)plcControls.FindControl("lnkPrev");
            LinkButton lnkNext = (LinkButton)plcControls.FindControl("lnkNext");
            LinkButton lnkLast = (LinkButton)plcControls.FindControl("lnkLast");

            if (lnkFirst == null) return; else lnkFirst.Enabled = false;
            if (lnkPrev == null) return; else lnkPrev.Enabled = false;
            if (lnkNext == null) return; else lnkNext.Enabled = false;
            if (lnkLast == null) return; else lnkLast.Enabled = false;  
            
            if (_totalPages==1) //If there is only single record then return
            {
                return;
            }
            if (curIndex==1) //First Record
            {
                lnkNext.Enabled = true;
                lnkLast.Enabled = true; 
            }
            else if (curIndex == _totalPages)
            {
                lnkFirst.Enabled = true;
                lnkPrev.Enabled = true;
            }
            else if (curIndex < _totalPages) // In between Record
            {
                lnkFirst.Enabled = true;
                lnkPrev.Enabled = true;
                lnkNext.Enabled = true;
                lnkLast.Enabled = true;
            }
        }
        #endregion

        #region ShowData
        private void BindData(int pageIndex)
        {
            CustomPaging.Class.TimeObj t1 = new CustomPaging.Class.TimeObj();
            int startRow;
            int endRow;

            startRow = (pageIndex * grdEmployee.PageSize) +1;
            endRow= startRow + grdEmployee.PageSize -1;
            t1.Start();
            grdEmployee.DataSource= CustomPaging.Class.Common.DataReaderToDataTable(SelectQuery,startRow,endRow);
            grdEmployee.DataBind();
            t1.Stop();
            lblReaderResult.Text = string.Format("Total Records :{0}<br/>", _totalRecords);
            lblReaderResult.Text = lblReaderResult.Text + string.Format("Total Time : <b>{0}</b> seconds", t1.TS.TotalSeconds.ToString());
        }        
        private void BindDataTable()
        {
            CustomPaging.Class.TimeObj t1 = new CustomPaging.Class.TimeObj();
            t1.Start();
            grdEmpDT.DataSource = CustomPaging.Class.Common.GetDataTable(SelectQuery);
            grdEmpDT.DataBind();
            t1.Stop();
            lblDataTableResult.Text = string.Format("Total Records :{0}<br/>", _totalRecords);
            lblDataTableResult.Text = lblDataTableResult.Text + string.Format("Total Time : <b>{0}</b> seconds", t1.TS.TotalSeconds.ToString());
        }
        private void Navigate(object sender, CommandEventArgs e)
        {
            int curIndex;
            curIndex = int.Parse(e.CommandArgument.ToString()) - 1;
            ViewState["CurIndex"] = int.Parse(e.CommandArgument.ToString());
            BindData(curIndex);
            EnableDisableLink(int.Parse(e.CommandArgument.ToString()));
            
            //Implement Second Grid Changing
            grdEmpDT.PageIndex = int.Parse(e.CommandArgument.ToString())-1;
            BindDataTable();
        }
        private int GetRecdCount()
        {
            DateTime t1;
            DateTime t2;
            TimeSpan ts;

            DataTable dt;
            int rowcount = 0;
            t1 = DateTime.Now;
            dt = CustomPaging.Class.Common.GetDataTable(RecCountQuery);
            rowcount = int.Parse(dt.Rows[0].ItemArray.GetValue(0).ToString());
            t2 = DateTime.Now;
            ts = t2.Subtract(t1);
            System.Diagnostics.Debug.WriteLine("GetRecordCount: " + ts.TotalSeconds);
            return rowcount;
        }
        #endregion

    }
}
